# Generated by Django 4.1.2 on 2022-12-09 00:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sphinxhostingcore', '0003_globaltoc'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sphinxpage',
            name='next_page',
            field=models.ForeignKey(help_text='The next page in the documentation set', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='previous_page', to='sphinxhostingcore.sphinxpage'),
        ),
    ]
