# Generated by Django 4.1.2 on 2023-01-19 19:21

from django.db import migrations
from django.core.management import call_command

fixture: str = 'classifiers'


def load_fixture(apps, schema_editor):
    call_command('loaddata', fixture, app_label='sphinxhostingcore')


def unload_fixture(apps, schema_editor):
    Classifier = apps.get_model("sphinxhostingcore", "Classifier")
    Classifier.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('sphinxhostingcore', '0006_classifiers_and_permissiongroups'),
    ]

    operations = [
        migrations.RunPython(load_fixture, reverse_code=unload_fixture),
    ]
