# Generated by Django 4.1.2 on 2023-02-03 23:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sphinxhostingcore', '0007_load_classifiers'),
    ]

    operations = [
        migrations.AddField(
            model_name='version',
            name='archived',
            field=models.BooleanField(default=False, help_text='Whether this version should be excluded from search indexes', verbose_name='Archived?'),
        ),
    ]
