{% extends 'academy_theme/base--wildewidgets.html' %}
{% load static %}

{% block extra_css %}
{{ block.super }}
<link rel="stylesheet" href="{% static 'sphinx_hosting/css/sphinx_hosting.css' %}">
{% endblock %}

{% block extra_footer_js %}
{{ block.super }}
<script src="https://cdnjs.cloudflare.com/ajax/libs/fslightbox/3.0.9/index.js" integrity="sha512-2VqLVM3WCyaqUgQb2hpoWHSus021RIN0Jq0wfrLqqLh+anm1kW/H4Yw7HVu3D5W4nbdUQmAA2mqQv2JEoy+kPA==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
{% endblock %}