# Generated by Django 4.1.2 on 2022-11-22 23:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sphinxhostingcore', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='sphinxpage',
            name='orig_body',
            field=models.TextField(blank=True, help_text='The original body for the page, extracted from the page JSON. Some pages have no body.We save this here in case we need to reprocess the body at some later date.', verbose_name='Body (Original'),
        ),
        migrations.AddField(
            model_name='sphinxpage',
            name='orig_local_toc',
            field=models.TextField(blank=True, default=None, help_text='The original table of contents for headings in this page.We save this here in case we need to reprocess the table of contents at some later date.', null=True, verbose_name='Local Table of Contents (original)'),
        ),
        migrations.AlterField(
            model_name='sphinxpage',
            name='body',
            field=models.TextField(blank=True, help_text='The body for the page, extracted from the page JSON, and modified to suit us.  Some pages have no body.', verbose_name='Body'),
        ),
        migrations.AlterField(
            model_name='sphinxpage',
            name='local_toc',
            field=models.TextField(blank=True, default=None, help_text='Table of Contents for headings in this page, modified to work in our templates', null=True, verbose_name='Local Table of Contents'),
        ),
    ]
